@@ -1,5 +1,7 @@
-use v6.c;
+use v6;
+
 use LibraryMake;
+
 use NativeCall;
 
 =begin pod
@@ -125,6 +127,9 @@ The enum C<UtmpType> defines the constants as:
 
 =item UserProcess - record created for a user process (e.g. a login )
 
+Some systems don't define all of these, in these cases they are defined with
+a value that is never usedl
+
 An object of L<Sys::Utmp::Utemp> can be smart matched to this values to
 select for items of a certain type.
 
@@ -144,9 +149,30 @@ This returns a L<DateTime> that corresponds to C<tv>
 
 =end pod
 
-class Sys::Utmp:ver<0.0.8>:auth<github:jonathanstowe> {
+class Sys::Utmp:ver<0.0.12>:auth<github:jonathanstowe>:api<1.0> {
 
-    enum UtmpType is export <EmptyRecord RunLevel BootTime NewTime OldTime InitProcess LoginProcess UserProcess DeadProcess Accounting>;
+    my constant HELPER = %?RESOURCES<libraries/utmphelper>.Str;
+
+    my sub _p_ut_constant(Str $constname --> int32) is native(HELPER) { * }
+
+    my sub ut_const(Str $constname --> Int) {
+        my $const = _p_ut_constant($constname);
+        if $const < 0 {
+            die "It seems the constant $constname isn't defined";
+        }
+        $const;
+    }
+    
+    enum UtmpType is export ( EmptyRecord   => ut_const('EMPTY_RECORD'),
+                              RunLevel      => ut_const('RUN_LVL'),
+                              BootTime      => ut_const('BOOT_TIME'),
+                              NewTime       => ut_const('NEW_TIME'),
+                              OldTime       => ut_const('OLD_TIME'),
+                              InitProcess   => ut_const('INIT_PROCESS'),
+                              LoginProcess  => ut_const('LOGIN_PROCESS'),
+                              UserProcess   => ut_const('USER_PROCESS'),
+                              DeadProcess   => ut_const('DEAD_PROCESS'),
+                              Accounting    => ut_const('ACCOUNTING'));
 
     class Utent is repr('CStruct') {
         has int8 $.type;
@@ -157,29 +183,23 @@ class Sys::Utmp:ver<0.0.8>:auth<github:jonathanstowe> {
         has Str $.host;
         has int $.tv;
 
-        method timestamp() {
+        method timestamp( --> DateTime ) {
             DateTime.new($!tv // 0 );
         }
 
-        method gist() {
-            $!user ~ "\t" ~ $!line ~ "\t" ~ $.timestamp;
+        method gist( --> Str ) {
+            "{ $!user }\t{ $!line }\t{ $.timestamp }\t{ $!host }";
         }
 
-        method Numeric() {
+        method Numeric( --> Numeric ) {
             $!type;
         }
 
-        multi method ACCEPTS(Utent:D: UtmpType $type) {
+        multi method ACCEPTS(Utent:D: UtmpType $type --> Bool ) {
             $!type == $type;
         }
     }
 
-    my constant HELPER = %?RESOURCES<libraries/utmphelper>.Str;
-
-    sub library {
-        my $lib = 'libraries/' ~ $*VM.platform-library-name('utmphelper'.IO).Str;
-        %?RESOURCES{$lib}.Str;
-    }
 
     my sub _p_setutent() is native(HELPER) { * }
 
@@ -201,9 +221,9 @@ class Sys::Utmp:ver<0.0.8>:auth<github:jonathanstowe> {
         _p_utmpname($n);
     }
 
-    my sub _p_getutent() returns Utent is native(HELPER) { * }
+    my sub _p_getutent( --> Utent ) is native(HELPER) { * }
 
-    method getutent() returns Utent {
+    method getutent( --> Utent ) {
         _p_getutent();
     }
 
